@@ -1,3 +1,5 @@
+require 'puppet'
+
 module Puppet
 module Util
 module Puppetdb
@@ -76,7 +78,18 @@ module CharEncoding
 
    def self.ruby18_clean_utf8(str)
      #iconv_to_utf8(str)
-     ruby18_manually_clean_utf8(str)
+     #ruby18_manually_clean_utf8(str)
+
+     # So, we've tried doing this UTF8 cleaning for ruby 1.8 a few different
+     # ways.  Doing it via IConv, we don't do a good job of handling characters
+     # whose codepoints would exceed the legal maximum for UTF-8.  Doing it via
+     # our manual scrubbing process is slower and doesn't catch overlong
+     # encodings.  Since this code really shouldn't even exist in the first place
+     # we've decided to simply compose the two scrubbing methods for now, rather
+     # than trying to add detection of overlong encodings.  It'd be a non-trivial
+     # chunk of code, and it'd have to do a lot of bitwise arithmetic (which Ruby
+     # is not blazingly fast at).
+     ruby18_manually_clean_utf8(iconv_to_utf8(str))
    end
 
 
@@ -93,27 +106,36 @@ module CharEncoding
      Utf8CharLens[byte]
    end
 
-   # Manually cleans a string by replacing any byte sequences that are not
-   #  valid UTF-8 characters with the unicode replacement character.  Due
-   #  to differences in how the [] operator works on strings in ruby 1.8 vs.
-   #  ruby 1.9, this method will NOT work with ruby 1.9.
-   def self.ruby18_manually_clean_utf8(str)
+   # Manually cleans a string by stripping any byte sequences that are
+   # not valid UTF-8 characters.  If you'd prefer for the invalid bytes to be
+   # replaced with the unicode replacement character rather than being stripped,
+   # you may pass `false` for the optional second parameter (`strip`, which
+   # defaults to `true`).
+   def self.ruby18_manually_clean_utf8(str, strip = true)
+
+     # This is a hack to allow this code to work with either ruby 1.8 or 1.9,
+     # which is useful for debugging and benchmarking.  For more info see the
+     # comments in the #get_byte method below.
+     @has_get_byte = str.respond_to?(:getbyte)
+
+
      i = 0
      len = str.length
      result = ""
 
      while i < len
-       byte = str[i]
+       byte = get_byte(str, i)
+
        i += 1
 
        char_len = get_char_len(byte)
        case char_len
        when 0
-         result.concat(Utf8ReplacementChar)
+         result.concat(Utf8ReplacementChar) unless strip
        when 1
          result << byte
        when 2..4
-         ruby18_handle_multibyte_char(result, byte, str, i,  char_len)
+         ruby18_handle_multibyte_char(result, byte, str, i,  char_len, strip)
          i += char_len - 1
        else
          raise Puppet::DevError, "Unhandled UTF8 char length: '#{char_len}'"
@@ -124,7 +146,7 @@ module CharEncoding
      result
    end
 
-  def self.ruby18_handle_multibyte_char(result_str, byte, str, i, char_len)
+  def self.ruby18_handle_multibyte_char(result_str, byte, str, i, char_len, strip = true)
     # keeping an array of bytes for now because we need to do some
     #  bitwise math on them.
     char_additional_bytes = []
@@ -132,7 +154,7 @@ module CharEncoding
     # If we don't have enough bytes left to read the full character, we
     #  put on a replacement character and bail.
     if i + (char_len - 1) > str.length
-      result_str.concat(Utf8ReplacementChar)
+      result_str.concat(Utf8ReplacementChar) unless strip
       return
     end
 
@@ -140,14 +162,14 @@ module CharEncoding
     #  from 0 to (n-2); e.g. if it's a 2-byte char, we will have a range
     #  from 0 to 0 which will result in reading 1 more byte
     (0..char_len - 2).each do |x|
-      char_additional_bytes << str[i + x]
+      char_additional_bytes << get_byte(str, i + x)
     end
 
     if (is_valid_multibyte_suffix(byte, char_additional_bytes))
       result_str << byte
       result_str.concat(char_additional_bytes.pack("c*"))
     else
-      result_str.concat(Utf8ReplacementChar)
+      result_str.concat(Utf8ReplacementChar) unless strip
     end
   end
 
@@ -165,6 +187,18 @@ module CharEncoding
     additional_bytes.all? { |b| ((b & 0xC0) == 0x80) }
   end
 
+  def self.get_byte(str, index)
+    # This method is a hack to allow this code to work with either ruby 1.8
+    #  or 1.9.  In production this code path should never be exercised by
+    #  1.9 because it has a much more sane way to accomplish our goal, but
+    #  for testing, it is useful to be able to run the 1.8 codepath in 1.9.
+    if @has_get_byte
+      str.getbyte(index)
+    else
+      str[index]
+    end
+  end
+
 end
 end
 end
