@@ -4,7 +4,7 @@
   (:import [org.apache.activemq.broker BrokerService]
            [org.apache.activemq ScheduledMessage]
            [org.apache.activemq.usage SystemUsage])
-  (:require [cheshire.core :as json]
+  (:require [com.puppetlabs.cheshire :as json]
             [clamq.activemq :as activemq]
             [clamq.protocol.connection :as mq-conn]
             [clamq.protocol.consumer :as mq-consumer]
@@ -205,10 +205,8 @@
                                     :on-message #(swap! contents conj %)
                                     :on-failure #(deliver mq-error (:exception %))})]
     (mq-consumer/start consumer)
-    (loop []
-      (when (> limit (count @contents))
-        (Thread/sleep 10)
-        (recur)))
+    (while (> limit (count @contents))
+      (Thread/sleep 10))
 
     (mq-consumer/close consumer)
     (if (realized? mq-error)
