@@ -1,10 +1,10 @@
 (ns com.puppetlabs.puppetdb.cli.anonymize
-  (:use [com.puppetlabs.utils :only (cli!)]
+  (:use [puppetlabs.kitchensink.core :only (cli!)]
         [com.puppetlabs.puppetdb.cli.export :only [export-root-dir export-metadata-file-name]]
         [com.puppetlabs.puppetdb.cli.import :only [parse-metadata]])
   (:import  [com.puppetlabs.archive TarGzReader TarGzWriter]
             [org.apache.commons.compress.archivers.tar TarArchiveEntry])
-  (:require [cheshire.core :as json]
+  (:require [com.puppetlabs.cheshire :as json]
             [clojure.java.io :as io]
             [clojure.string :as string]
             [com.puppetlabs.archive :as archive]
