@@ -41,13 +41,14 @@
   (:require [clojure.tools.logging :as log]
             [com.puppetlabs.puppetdb.catalogs :as cat]
             [com.puppetlabs.puppetdb.catalog.utils :as catutils]
+            [com.puppetlabs.utils.logging :as logutils]
             [com.puppetlabs.puppetdb.command :as command]
             [com.puppetlabs.http :as pl-http]
-            [cheshire.core :as json]
+            [com.puppetlabs.cheshire :as json]
             [clj-http.client :as client]
             [clj-http.util :as util]
             [fs.core :as fs])
-  (:use [com.puppetlabs.utils :only (cli! inis-to-map configure-logging! utf8-string->sha1)]
+  (:use [puppetlabs.kitchensink.core :only (cli! inis-to-map utf8-string->sha1)]
         [com.puppetlabs.puppetdb.scf.migrate :only [migrate!]]
         [com.puppetlabs.puppetdb.command.constants :only [command-names]]))
 
@@ -185,7 +186,7 @@
   (let [[options _]     (cli! args supported-cli-options required-cli-options)
         config          (-> (:config options)
                             (inis-to-map)
-                            (configure-logging!))
+                            (logutils/configure-logging!))
 
         catalogs        (if (:catalogs options) (load-sample-data (:catalogs options)))
         reports         (if (:reports options) (load-sample-data (:reports options)))
