@@ -8,7 +8,7 @@
   (app
    [&]
    {:get (comp (fn [{:keys [params globals] :as request}]
-                 (v3-facts/query-facts (params "query") {} (:scf-db globals)))
+                 (v3-facts/query-facts (params "query") {} (:scf-read-db globals)))
                http-q/restrict-query-to-active-nodes)}))
 
 (def facts-app
